@@ -20,10 +20,13 @@
 import abc
 import logging
 from io import StringIO
+from typing import Optional
 
+from pydantic import ValidationError
 from toolz import count, pipe
 
 from .. import datasets
+from ..releases import Releases
 from ..exceptions import StorageError
 from ..observations import iter_tsv_decoded_rows, iter_tsv_rows
 
@@ -82,6 +85,27 @@ class AbstractStorage(abc.ABC):
     def load_provider_json(self):
         pass
 
+    @abc.abstractmethod
+    def load_releases_json(self):
+        """Return `releases.json` content. Since the file is optional, return `None` if not found."""
+        pass
+
+    def load_releases(self) -> Optional[Releases]:
+        """Load releases metadata."""
+        releases_json = self.load_releases_json()
+        if releases_json is None:
+            return None
+
+        try:
+            releases = Releases.parse_obj(releases_json)
+        except ValidationError as exc:
+            raise StorageError(
+                f"Invalid releases for provider {self.provider_code}",
+                provider_code=self.provider_code,
+            ) from exc
+
+        return releases
+
     @property
     @abc.abstractmethod
     def path(self):
