@@ -67,6 +67,9 @@ class FileSystemStorage(AbstractStorage):
             raise StorageError("Could not load \"{}\"".format(filename), self.provider_code)
         return provider_json
 
+    def load_releases_json(self):
+        return load_json_file(self, self.path / "releases.json")
+
     @property
     def path(self):
         return self.storage_dir_path
