@@ -31,11 +31,17 @@ from collections import defaultdict
 from pathlib import Path
 
 import jsonschema.exceptions
+from pydantic import ValidationError
 from toolz import get, take
 
 from dbnomics_data_model import storages, validators
 from dbnomics_data_model.exceptions import StorageError
-from dbnomics_data_model.observations import NOT_AVAILABLE, detect_period_format_strict, value_to_float
+from dbnomics_data_model.observations import (
+    NOT_AVAILABLE,
+    detect_period_format_strict,
+    value_to_float,
+)
+from dbnomics_data_model.releases import Releases, parse_dataset_release, LATEST_RELEASE
 
 log = logging.getLogger(__name__)
 
@@ -121,6 +127,12 @@ def main():
             errors_codes_counts[error['error_code']] += 1
             print(format_error(error, output_format=args.format))
 
+        log.debug("Validating releases ...")
+        releases_errors = validate_releases(storage, ignore_errors=ignore_errors)
+        for error in releases_errors:
+            errors_codes_counts[error['error_code']] += 1
+            print(format_error(error, output_format=args.format))
+
         log.debug("Validating datasets...")
         nb_datasets = storage.get_nb_datasets()
         for dataset_index, dataset_dir in enumerate(storage.iter_datasets_dirs(sort_by_dir_name=True), start=1):
@@ -209,7 +221,9 @@ def format_error(error, output_format="text"):
         for cause_error in cause:
             sio.write("    - at path: {}\n".format(cause_error["path"]))
             sio.write("      message: {}\n".format(cause_error["message"]))
-            sio.write("      value: {!r}\n".format(cause_error["value"]))
+            value = cause_error.get("value")
+            if value is not None:
+                sio.write("      value: {!r}\n".format(value))
 
     return sio.getvalue()
 
@@ -226,11 +240,21 @@ def build_jsonschema_error(errors, base_path=[]):
     ]
 
 
-def category_tree_dataset_code_iter(category_tree):
+def cause_from_validation_error(exc: ValidationError):
+    return [
+        {
+            "path": error["loc"],
+            "message": error["msg"],
+        }
+        for error in exc.errors()
+    ]
+
+
+def iter_category_tree_dataset_code(category_tree):
     """Yield all dataset codes from a category_tree instance"""
     for elt in category_tree:
         if 'children' in elt:
-            for dataset_code in category_tree_dataset_code_iter(elt['children']):
+            for dataset_code in iter_category_tree_dataset_code(elt['children']):
                 yield dataset_code
         elif elt.get('code'):
             yield elt['code']
@@ -257,8 +281,8 @@ def validate_category_tree(storage, ignore_errors=[]):
                 })
 
     if category_tree_json is not None:
-        dataset_codes_on_storage = set([dataset.dataset_code for dataset in storage.iter_datasets_dirs()])
-        datasets_codes_in_category_tree = set(list(category_tree_dataset_code_iter(category_tree_json)))
+        dataset_codes_on_storage = set(dataset.dataset_code for dataset in storage.iter_datasets_dirs())
+        datasets_codes_in_category_tree = set(iter_category_tree_dataset_code(category_tree_json))
 
         # Check that all datasets referenced in category_tree.json are present on disk
         error_code = "dataset-not-found-on-storage"
@@ -305,6 +329,49 @@ def validate_category_tree(storage, ignore_errors=[]):
     return errors
 
 
+def validate_releases(storage, ignore_errors=[]):
+    provider_code = storage.provider_code
+    errors = []
+
+    error_code = "invalid-releases"
+    if error_code not in ignore_errors:
+        log.debug("Validating releases.json...")
+        try:
+            releases = storage.load_releases()
+        except ValidationError as exc:
+            errors.append({
+                "cause": cause_from_validation_error(exc),
+                "error_code": error_code,
+                "message": "Invalid releases",
+                "provider_code": provider_code,
+                "location": "releases.json",
+            })
+
+    # Check that all datasets referenced in releases exist
+    error_code = "dataset-not-found-on-storage"
+    if error_code not in ignore_errors and releases is not None:
+        dataset_codes_on_storage = {
+            dataset.dataset_code
+            for dataset in storage.iter_datasets_dirs()
+        }
+        datasets_codes_in_releases = {
+            dataset_releases_item.format_release(release.code)
+            for dataset_releases_item in releases.dataset_releases
+            for release in dataset_releases_item.releases
+        }
+        not_found_on_storage = datasets_codes_in_releases - dataset_codes_on_storage
+        for dataset_code in not_found_on_storage:
+            errors.append({
+                "error_code": error_code,
+                "message": f"Dataset {dataset_code!r} declared in releases.json is not found on storage",
+                "provider_code": provider_code,
+                "dataset_code": dataset_code,
+                "location": "releases.json"
+            })
+
+    return errors
+
+
 def validate_dataset(dataset_dir, ignore_errors=[]):
     errors = []
     provider_code = dataset_dir.storage.provider_code
@@ -383,6 +450,25 @@ def validate_dataset(dataset_dir, ignore_errors=[]):
             "location": dataset_code,
         })
 
+    # Check that release codes are different that "latest"
+    error_code = "invalid-release-code"
+    if error_code not in ignore_errors:
+        is_invalid = False
+        try:
+            _, release_code = parse_dataset_release(dataset_code)
+        except ValueError:
+            is_invalid = True
+        else:
+            is_invalid = release_code == LATEST_RELEASE
+        if is_invalid:
+            errors.append({
+                "error_code": error_code,
+                "message": f"Dataset {dataset_code!r} has an invalid release code {LATEST_RELEASE!r}",
+                "provider_code": provider_code,
+                "dataset_code": dataset_code,
+                "location": f"{dataset_code}/dataset.json",
+            })
+
     return (dataset_json, dataset_series, errors)
 
 
